#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdio.h>

#include "newt.h"
#include "newt_pr.h"

struct label {
    char * text;
    int length;
    GtkWidget *widget;
    GtkWidget *parent;
};

extern GtkWidget* currentParent;
extern int FontSizeW;
extern int FontSizeH;
extern int ScreenW;
extern int ScreenH;
extern struct newtColors * globalColors;
extern GdkColor getColorByName (gchar *name);

static void labelDraw(newtComponent co);
static void labelDestroy(newtComponent co);

static struct componentOps labelOps = {
    labelDraw,
    newtDefaultEventHandler,
    labelDestroy,
    newtDefaultPlaceHandler,
    newtDefaultMappedHandler,
} ;

newtComponent newtLabel(int left, int top, const char * text) {
    newtComponent co;
    struct label * la;

    co = malloc(sizeof(*co));
    la = malloc(sizeof(struct label));
    co->data = la;

    co->ops = &labelOps;

    co->height = 1;
    co->width = strlen(text);
    co->top = top;
    co->left = left;
    co->takesFocus = 0;

    la->length = strlen(text);
    la->text = strdup(text);
    la->widget = gtk_label_new (text);
    la->parent = NULL;

    return co;
}

void newtLabelSetText(newtComponent co, const char * text) {
    int newLength;
    struct label * la = co->data;

    newLength = strlen(text);
    if (newLength <= la->length) {
	memset(la->text, ' ', la->length);
	memcpy(la->text, text, newLength);
    } else {
	free(la->text);
	la->text = strdup(text);
	la->length = newLength;
	co->width = newLength;
    }

    labelDraw(co);
}

static void labelDraw(newtComponent co) {
    struct label * la = co->data;
    GtkStyle *style;
    char fnt[100];
    int fnt_size;

    if (co->top == -1) return;

    if (!la->parent) {
	la->parent = currentParent;
	style = gtk_style_copy (gtk_widget_get_style (la->widget));
	style->fg[0] = getColorByName (globalColors->labelFg);
	gdk_font_unref (style->font);
	fnt_size = (int)((float)FontSizeH * 0.75) - 2;
	if (FontSizeH < 18) {
		fnt_size += 2;
	}
	snprintf ( fnt, 100, 
		"-adobe-courier-medium-r-normal-*-*-%d0-*-*-m-*-iso8859-1",
		fnt_size);
	style->font = gdk_font_load (fnt);
	gtk_widget_set_style (la->widget, style);
	gtk_widget_show (la->widget);
        gtk_fixed_put (GTK_FIXED (la->parent), la->widget,
            FontSizeW * co->left, FontSizeH * co->top);
    }
    gtk_label_set_text ((GtkLabel*) la->widget, la->text);
}

static void labelDestroy(newtComponent co) {
    struct label * la = co->data;

    free(la->text);
    free(la);
    free(co);
}
